@@ -72,7 +72,6 @@ outline_window_expose (GtkWidget      *widget,
                        gpointer        data)
 {
   MetaTabPopup *popup;
-  int w, h;
   TabEntry *te;  
   
   popup = data;
@@ -82,8 +81,6 @@ outline_window_expose (GtkWidget      *widget,
 
   te = popup->current_selected_entry;
   
-  gdk_window_get_size (widget->window, &w, &h);
-
   gdk_draw_rectangle (widget->window,
                       widget->style->white_gc,
                       FALSE,
@@ -101,20 +98,6 @@ outline_window_expose (GtkWidget      *widget,
   return FALSE;
 }
 
-static char*
-utf8_strndup (const char *src,
-              int         n)
-{
-  const gchar *s = src;
-  while (n && *s)
-    {
-      s = g_utf8_next_char (s);
-      n--;
-    }
-
-  return g_strndup (src, s - src);
-}
-
 static GdkPixbuf*
 dimm_icon (GdkPixbuf *pixbuf)
 {
@@ -241,7 +224,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
               tmp = markup;
             }
             
-          te->title = utf8_strndup (tmp, max_chars_per_title);
+          te->title = meta_g_utf8_strndup (tmp, max_chars_per_title);
           g_free (tmp);
         }
       te->widget = NULL;
@@ -488,8 +471,11 @@ display_entry (MetaTabPopup *popup,
       gdk_region_destroy (region);
   
       /* This should piss off gtk a bit, but we don't want to raise
-       * above the tab popup
+       * above the tab popup.  So, instead of calling gtk_widget_show,
+       * we manually set the window as mapped and then manually map it
+       * with gdk functions.
        */
+      GTK_WIDGET_SET_FLAGS (popup->outline_window, GTK_MAPPED);
       gdk_window_show_unraised (popup->outline_window->window);
     }
 
